//================================================================
//MARK: Globals
//================================================================

boolean kRotate90 = true;

int kBodyCount = 4;
float kBodySpacing = 250.0;
int kSegmentCount = 50; //50

float kPieceTimeInSeconds = 1 * 60;

float kSegmentMinimumLength = 20;
float kSegmentMaximumLength = 60;

float kMaxRandomWalkWidth = 100;

float kBodyTendrilStartFadeLength = 300;
float kBodyTendrilEndFadeLength = 300;

float kMinimumTendrilCountPerSegment = 0;
float kMaximumTendrilCountPerSegment = 20;

float kMinimumTendrilLength = 50;
float kMaximumTendrilLength = 350;

float kSegmentStrokeWeight = 4;
float kTendrilStrokeWeight = 2;

float kSpeedMultiplier = 4.0;
float kMaximumCameraZoom = 0.525; //0.525 for 5

float kRestRampTime = 900.0;
float kActiveStartTime;

//================================================================
//MARK: Interface
//================================================================

boolean kActive; //toggled to start
boolean kShowInstructions;
String kInstructionsString = "i. each player follows one ink creature\nii. when circle rune is showing, players must rest\niii. lines without tendrils are a single pitch\niv. tendrils are accents, timbre articulations, and wildly variable pitch\nv. dynamics should never go above piano";

//================================================================
//MARK: Colors
//================================================================

color kSegmentColor = color(50, 48, 30); //rgb
color kBGColor = color(215, 215, 175); //rgb

color kCornerTintLight;
color kCornerTintDark;

color[] kBodyColorsHSB = { //rgb here are representing hsb, set in setup()
  color(208.0 / 360 * 255, 0.01 * 255, 0.45 * 255), 
  color(42.0 / 360 * 255, 0.10 * 255, 0.65 * 255), 
  color(240.0 / 360 * 255, 0.0 * 255, 0.82 * 255), 
  color(55.0 / 360 * 255, 0.22 * 255, 0.64 * 255), 
  color(49.0 / 360 * 255, 0.43 * 255, 0.65 * 255)
};


//================================================================
//MARK: Enums
//================================================================

enum Side { 
  LEFTSIDE, RIGHTSIDE
}

//================================================================
//MARK: Bezier Functions
//================================================================

void bezierTentacleWithPoints(float[] points, float thickness, PGraphics pg) {
  pg.beginShape();
  pg.beginShape();
  pg.vertex(points[0], points[1]);
  pg.bezierVertex(points[2], points[3], points[4], points[5], points[6], points[7]);
  pg.bezierVertex(points[4], points[5], points[2], points[3], points[0], points[1] + thickness);
  pg.endShape();
}

void bezierWithPoints(float[] points) {
  bezier(
    points[0], 
    points[1], 
    points[2], 
    points[3], 
    points[4], 
    points[5], 
    points[6], 
    points[7]
    );
}

void bezierWithPoints(float[] points, PGraphics pg) {
  pg.bezier(
    points[0], 
    points[1], 
    points[2], 
    points[3], 
    points[4], 
    points[5], 
    points[6], 
    points[7]
    );
}

float[] reverseBezierPoints(float[] points) {
  float[] r = new float[points.length];
  r[0] = points[6];
  r[1] = points[7];
  r[2] = points[4];
  r[3] = points[5];
  r[4] = points[2];
  r[5] = points[3];
  r[6] = points[0];
  r[7] = points[1];
  return r;
}

float[] splitBezier(float[] points, float t) {
  if (t == 1.0) {
    return points;
  }
  float x1, y1, x2, y2, x3, y3, x4, y4 = 0.0;
  x1 = points[0];
  y1 = points[1];
  x2 = points[2];
  y2 = points[3];
  x3 = points[4];
  y3 = points[5];
  x4 = points[6];
  y4 = points[7];

  float x12 = (x2-x1)*t+x1;
  float y12 = (y2-y1)*t+y1;

  float x23 = (x3-x2)*t+x2;
  float y23 = (y3-y2)*t+y2;

  float x34 = (x4-x3)*t+x3;
  float y34 = (y4-y3)*t+y3;

  float x123 = (x23-x12)*t+x12;
  float y123 = (y23-y12)*t+y12;

  float x234 = (x34-x23)*t+x23;
  float y234 = (y34-y23)*t+y23;

  float x1234 = (x234-x123)*t+x123;
  float y1234 = (y234-y123)*t+y123;

  float[] r = {
    x1, y1, x12, y12, x123, y123, x1234, y1234
  };
  return r;
}

float[] splitBezier(float[] points, float t0, float t1) {
  if (t0 > t1) {
    return new float[8];
  }

  float[] r =  splitBezier(points, t1);
  float[] rev = reverseBezierPoints(r);
  float t = (t1 - t0) / t1;
  r = splitBezier(rev, t);
  return r;
}