//================================================================
//MARK: Rune
//================================================================

class Rune {

  //-------------------------------------------------
  //Private Properties
  //-------------------------------------------------

  private float maxDiameter;
  private PVector center;
  private color runeColor;

  private Ramp scaleRamp, rotationRamp;

  private PGraphics pg;
  private boolean showing;

  //-------------------------------------------------
  //Readonly Properties
  //-------------------------------------------------

  boolean isShowing() {
    return showing;
  }

  //-------------------------------------------------
  //Initializer
  //-------------------------------------------------
  Rune(float diameter, color r) {
    maxDiameter = diameter;
    center = new PVector(0, 0);
    scaleRamp = new Ramp();
    scaleRamp.mode = RampMode.RAMP_MODE_S_CURVE;
    rotationRamp = new Ramp();
    rotationRamp.mode = RampMode.RAMP_MODE_S_CURVE;
    runeColor = r;

    showing = false;

    pg = createGraphics(int(maxDiameter) + 10, int(maxDiameter) + 10, P2D); //10 to allow for strokeWeight / other edges
    drawGraphics();
  }

  void updateDraw(PGraphics p) {
    float scale = scaleRamp.update();
    if (scale == 0.0 && scaleRamp.isFinished()) {
      showing = false;
      return;
    }
    float rotation = rotationRamp.update() * PI;
    float generalRotation = float(frameCount) * 0.002;

    if (p != null) {
      p.pushMatrix();
      p.translate(center.x, center.y);
      p.rotate(rotation + generalRotation);
      p.scale(scale);
      p.image(pg, -pg.width / 2, -pg.height / 2);
      p.popMatrix();
    } else {
      pushMatrix();
      translate(center.x, center.y);
      rotate(rotation + generalRotation);
      scale(scale);
      image(pg, -pg.width / 2, -pg.height / 2);
      popMatrix();
    }
  }

  void showAtPosition(PVector p) {
    if (showing) {
      return;
    }
    center = p;
    scaleRamp.rampToValueOverDuration(1.0, 1200);
    rotationRamp.rampToValueOverDuration(1.0, 1600);
    showing = true;
  }

  void hide() {
    if (!showing) {
      return;
    }
    scaleRamp.rampToValueOverDuration(0.0, 600);
    rotationRamp.rampToValueOverDuration(2.0, 800);
  }

  private void drawGraphics() {
    pg.beginDraw();
    pg.pushMatrix();
    pg.translate(pg.width / 2, pg.height / 2);
    pg.stroke(runeColor);
    pg.noFill();
    //pg.fill(runeColor);
    pg.strokeWeight(2);

    float theta = random(PI * 2.0);

    ArrayList<PVector> centers = new ArrayList<PVector>();

    //pick a starting edge
    float radius = 5.0;
    for (int n=0; n<3; n++) {
      PVector start = new PVector(cos(theta) * (100.0 - radius), sin(theta) * (100.0 - radius));
      centers.add(start);
      pg.ellipse(start.x, start.y, radius * 2, radius * 2);
      for (int i=0; i<100; i++) {
        //get last point
        PVector last = centers.get(centers.size() - 1);
        //get a random position within the circle
        PVector rand = new PVector(
          cos(random(PI * 2.0)) * 100, 
          sin(random(PI * 2.0)) * 100
          );
        PVector next = new PVector(
          last.x + cos(theta) * radius * 2.0, 
          last.y + sin(theta) * radius * 2.0
          );
        //get the angle to that point
        theta = atan2(rand.y - last.y, rand.x - last.x);
        //hit test with others
        boolean hitOther = false;
        for (int j=0; j<centers.size(); j++) {
          PVector c = centers.get(j);
          float dc = next.dist(c);
          if (dc < radius * 2) {
            hitOther = true;
          }
        }
        if (next.dist(new PVector(0, 0)) < (100.0 - radius) &&
          !hitOther) {
          pg.stroke(runeColor, random(256));
          pg.noFill();
          pg.fill(runeColor, random(256));
          pg.ellipse(next.x, next.y, radius * 2, radius * 2);
          centers.add(next);
        }
      }
    }

    pg.stroke(runeColor);
    pg.noFill();
    pg.ellipse(0, 0, 200, 200);

    int circleCount = 100;
    float dTheta = (PI * 2.0) / float(circleCount);
    for (int i=0; i<circleCount; i++) {
      if (random(3) > 1.0) {
        pg.noStroke();
        pg.stroke(runeColor);
        pg.fill(0);
        PVector p = new PVector(cos(dTheta * i) * 100.0, sin(dTheta * i) * 100);
        pg.ellipse(p.x, p.y, 6, 6);
      }
    }
    pg.popMatrix();
    pg.endDraw();
  }
}